/*!
 * © 2022 David Vines
 *
 * domainOfTheAncients is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * any later version.
 *
 * domainOfTheAncients is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with domainOfTheAncients. If not, see https://www.gnu.org/licenses/gpl-2.0.html.
 */
/* globals Util, Game, TECHS, Map */
/* exported SensorOrders */
let SensorOrders = {
	showReports: function(player,turn) {
		let sensors = document.getElementById("sensors");
		player.map.forEachHex(function(hex) {
			if (hex.owner == player.stylename) {
				const comms = player.researchLevel(TECHS.communications);
				const range = (comms == 8 ? 40 : comms);
				const buildings = (comms == 8 ? 6 : Math.ceil(comms/2));
				for(let i=0; i<hex.sensors*buildings; i++) {
					const newrow = sensors.insertRow();
					const rowname = document.createElement("td"); rowname.append(hex.id);
					const target = document.createElement("td");
					const reportCell = document.createElement("td");
					let report = "None";
					if (hex.sensorTargets && hex.sensorTargets[i]) {
						const targetId = hex.sensorTargets[i];
						target.append(targetId);
						player.map.setLastScanned(targetId,turn);
						const targetHex = player.map.hex(targetId);
						if (targetHex.planet && targetHex.owner) {
							report = `${targetId} is a class ${targetHex.level} colony belonging to ${Game.FULLNAMES[targetHex.owner]}`;
						} else if (targetHex.planet) {
							report = `${targetId} is an uncolonised world`;
						}
						else {
							report = `${targetId} is an empty hex`;
						}
					} else {
						target.append("No Target");
					}
					reportCell.append(report);
					const changecell = document.createElement("td");
					const change = document.createElement("button");
					let which = {range: range, map: player.map, hex:hex.id, index:i, cell: target};
					change.classList.add("map-using");
					change.onclick = Util.captureExceptionFunction(SensorOrders._changeTarget,which);
					change.append("Change Target...");
					changecell.append(change);
					newrow.append(rowname,target,reportCell,changecell);
				}
			}
		});
	},
	_changeTarget: function(which) {
		Util.disableMapUsingButtons(true);
		document.getElementById("sensorinstructions").classList.remove("hidden");
		const cancel = document.getElementById("cancelScanChange");
		cancel.onclick = Util.captureExceptionFunction(SensorOrders._finishSelectTarget);

		for(let row=1; row<21; row++) {
			for(let column of "ABCDEFGHIJKLMNOP") {
				const name = column+row;
				const hexelement = document.getElementById(name);
				if (Map.range(which.hex,name) <= which.range) {
					const args = Object.assign({target: name},which);
					hexelement.onclick = Util.captureExceptionFunction(SensorOrders._selectTarget,args);
				} else {
					hexelement.classList.add("greyout");
				}
			}
		}
	},
	_selectTarget: function(which) {
		const sensorAt = which.hex;
		const sensorIndex = which.index;
		const target = which.target;
		const cell = which.cell;

		which.map[sensorAt].sensorTargets = which.map[sensorAt].sensorTargets ? which.map[sensorAt].sensorTargets : [];
		which.map[sensorAt].sensorTargets[sensorIndex] = target;
		cell.replaceChildren();
		cell.append(target);
		SensorOrders._finishSelectTarget();
	},
	_finishSelectTarget: function() {
		document.getElementById("sensorinstructions").classList.add("hidden");
		const cancel = document.getElementById("cancelScanChange");

		Util.disableMapUsingButtons(false);

		cancel.onclick = null;
		for(let row=1; row<21; row++) {
			for(let column of "ABCDEFGHIJKLMNOP") {
				const name = column+row;
				const hexelement = document.getElementById(name);
				hexelement.onclick = null;
				hexelement.classList.remove("greyout");
			}
		}

	}
};